@@ -12,7 +12,7 @@
 # See the file 'Changes' in the distrution archive.                         #
 #                                                                           #
 #############################################################################
-# 	$Id: Radius.pm,v 1.20 2009/07/23 12:27:46 psv Exp $
+# 	$Id: Radius.pm,v 1.21 2009/09/02 11:27:29 psv Exp $
 
 package Authen::Radius;
 
@@ -32,11 +32,11 @@ require Exporter;
 @EXPORT = qw(ACCESS_REQUEST ACCESS_ACCEPT ACCESS_REJECT
 			 ACCOUNTING_REQUEST ACCOUNTING_RESPONSE ACCOUNTING_STATUS
 			DISCONNECT_REQUEST);
-$VERSION = '0.14';
+$VERSION = '0.15';
 
 my (%dict_id, %dict_name, %dict_val, %dict_vendor_id, %dict_vendor_name );
 my ($request_id) = $$ & 0xff;	# probably better than starting from 0
-my ($radius_error) = 'ENONE';
+my ($radius_error, $error_comment) = ('ENONE', '');
 my $debug = 0;
 
 #
@@ -93,7 +93,7 @@ sub new {
 				Type => SOCK_DGRAM,
 				Proto => 'udp',
 				TimeOut => $self->{'timeout'}
-	) or return $self->set_error('ESOCKETFAIL');
+	) or return $self->set_error('ESOCKETFAIL', $@);
 
 	$self;
 }
@@ -120,7 +120,7 @@ sub send_packet {
 		print STDERR "Sending request:\n";
 		print STDERR HexDump($data);
 	}
-	$self->{'sock'}->send($data) || $self->set_error('ESENDFAIL');
+	$self->{'sock'}->send($data) || $self->set_error('ESENDFAIL', $!);
 }
 
 sub recv_packet {
@@ -130,9 +130,9 @@ sub recv_packet {
 	$self->set_error;
 
 	$sh = new IO::Select($self->{'sock'}) or return $self->set_error('ESELECTFAIL');
-	$sh->can_read($self->{'timeout'}) or return $self->set_error('ETIMEOUT');
+	$sh->can_read($self->{'timeout'}) or return $self->set_error('ETIMEOUT', $!);
 
-	$self->{'sock'}->recv ($data, 65536) or return $self->set_error('ERECVFAIL');
+	$self->{'sock'}->recv ($data, 65536) or return $self->set_error('ERECVFAIL', $!);
 	if ($debug) {
 		print STDERR "Received response:\n";
 		print STDERR HexDump($data);
@@ -397,17 +397,21 @@ sub load_dictionary {
 }
 
 sub set_error {
-	my ($self, $error) = @_;
-
-	$radius_error = $self->{'error'} = defined $error ? $error : 'ENONE';
-
+	my ($self, $error, $comment) = @_;
+    $@ = undef;
+	$radius_error = $self->{'error'} = (defined($error) ? $error : 'ENONE');
+    $error_comment = $self->{'error_comment'} = (defined($comment) ? $comment : '');
 	undef;
 }
 
 sub get_error {
 	my ($self) = @_;
 
-	$self->{'error'};
+    if (!ref($self)) {
+	    return $radius_error;
+    } else {
+    	return $self->{'error'};
+    }
 }
 
 sub strerror {
@@ -426,10 +430,21 @@ sub strerror {
 		'EBADID',	'response to unknown request'
 	);
 
-	return $errors{$radius_error} unless ref($self);
-	$errors{defined $error ? $error : $self->{'error'}};
+    if (!ref($self)) {
+	    return $errors{$radius_error};
+    }
+	return $errors{ (defined($error) ? $error : $self->{'error'} ) };
 }
 
+sub error_comment {
+	my ($self) = @_;
+
+    if (!ref($self)) {
+	    return $error_comment;
+    } else {
+    	return $self->{'error_comment'};
+    }
+}
 
 1;
 __END__
@@ -569,6 +584,11 @@ strings always beginning with an 'C<E>'.
 Returns a verbose error string for the last error for the current object, or
 for the specified C<ERRORCODE>.
 
+=item error_comment
+
+Returns the last error explanation for the current object. Error explanation 
+is generated by system call.
+
 =back
 
 =head1 AUTHOR
